/********************************************************************
 FileName:		main.c
 Dependencies:	See INCLUDES section
 Processor:		PIC18, PIC24, and PIC32 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 				boards supported by the MCHPFSUSB stack.  See release notes for
 				support matrix.  This demo can be modified for use on other hardware
 				platforms.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24), C32 (for PIC32)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.


********************************************************************
 File Description:

 Change History:
  Rev   Description
  ----  -----------------------------------------
  2.6a  Added support for PIC24FJ256GB210

  2.7   No change

  2.8   Improvements to USBCBSendResume(), to make it easier to use.
  2.9   Added event transfer terminated handler code, for improved MSD
        error case handling.  Increased RAM reserved for the C software 
        stack on PIC18 devices.  The previous version did not allocate
        enough RAM for the worst case scenario, when USB_INTERRUPTS mode
        was used, causing potentially unexpected operation.

 GD 29.10.2017
	Generatore di funzioni USB
			
********************************************************************/

/** INCLUDES *******************************************************/

#include <timer.h>
#include <ports.h>
#include <outcompare.h>
#include <ppsnew.h>
#include <i2c.h>
#include "GenericTypedefs.h"
#include "swi2c.h"
//#include "wdt.h"
//#include "PwrMgnt.h"

#include "USB.h"
#include "HardwareProfile.h"
#include "usb_config.h"

#include <libpic30.h>

#include "usb_function_hid.h"
#include "usb_gen_funz.h"
#include "dee emulation 16-bit.h"


/** CONFIGURATION **************************************************/

#ifdef __XC16__

// PIC24EP512GP202 Configuration Bit Settings
// 'C' source line config statements

// FICD
#pragma config ICS = PGD1               // ICD Communication Channel Select bits (Communicate on PGEC1 and PGED1)
#pragma config JTAGEN = OFF             // JTAG Enable bit (JTAG is disabled)

// FPOR
#pragma config ALTI2C1 = OFF            // Alternate I2C1 pins (I2C1 mapped to SDA1/SCL1 pins)
#pragma config ALTI2C2 = OFF            // Alternate I2C2 pins (I2C2 mapped to SDA2/SCL2 pins)
#pragma config WDTWIN = WIN25           // Watchdog Window Select bits (WDT Window is 25% of WDT period)

// FWDT
#pragma config WDTPOST = PS4096         // Watchdog Timer Postscaler bits (1:4,096)
#pragma config WDTPRE = PR32            // Watchdog Timer Prescaler bit (1:32)
#pragma config PLLKEN = ON              // PLL Lock Enable bit (Clock switch to PLL source will wait until the PLL lock signal is valid.)
#pragma config WINDIS = OFF             // Watchdog Timer Window Enable bit (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = ON              // Watchdog Timer Enable bit (Watchdog timer always enabled)

// FOSC
#pragma config POSCMD = NONE            // Primary Oscillator Mode Select bits (Primary Oscillator disabled)
#pragma config OSCIOFNC = ON            // OSC2 Pin Function bit (OSC2 is general purpose digital I/O pin)
#pragma config IOL1WAY = ON             // Peripheral pin select configuration (Allow only one reconfiguration)
#pragma config FCKSM = CSDCMD           // Clock Switching Mode bits (Both Clock switching and Fail-safe Clock Monitor are disabled)

// FOSCSEL
#pragma config FNOSC = FRCPLL           // Oscillator Source Selection (Fast RC Oscillator with divide-by-N with PLL module (FRCPLL))
#pragma config IESO = ON                // Two-speed Oscillator Start-up Enable bit (Start up device with FRC, then switch to user-selected oscillator source)

// FGS
#pragma config GWRP = OFF               // General Segment Write-Protect bit (General Segment may be written)
#pragma config GCP = OFF                // General Segment Code-Protect bit (General Segment Code protect is Disabled)
#endif
#if defined(__PIC24EP512GU810__)		// 
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF ) 
_FOSCSEL(FNOSC_PRIPLL & IESO_OFF)			//FNOSC_FRCPLL 
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSDCMD )
_FWDT( WDTPOST_PS4096 & WDTPRE_PR32 & PLLKEN_ON & WINDIS_OFF & FWDTEN_ON )
_FPOR( FPWRT_PWR8 & BOREN_ON & ALTI2C1_OFF & ALTI2C2_OFF )
_FICD( ICS_PGD3 & RSTPRI_PF & JTAGEN_OFF )		// 
_FAS( AWRP_OFF & APL_OFF & APLK_OFF)
//_FUID0(x) simpa..
#endif


/** VARIABLES ******************************************************/
#pragma udata


static const char CopyrString[]= {'U','S','B',' ','F','u','n','c','t','i','o','n',' ','g','e','n','e','r','a','t','o','r',' ',
	'v',VERNUMH+'0','.',VERNUML/10+'0',(VERNUML % 10)+'0', ' ','0','2','/','1','1','/','1','7', 0 };


const WORD TRIANGLE_WAVE_TABLE[NUM_SAMPLES]={0x2222,0x4444,0x6666,0x8888, 0xaaaa,0xcccc,0xeeee,0xffff,
	0xeeee,0xcccc,0xaaaa,0x8888, 0x6666,0x4444,0x2222,0x0000};
const WORD SINE_WAVE_TABLE[NUM_SAMPLES]={0x8000,0xb0fb,0xda82,0xf641, 0xffff,0xf641,0xda82,0xb0fb,
	0x8000,0x4f04,0x257d,0x09be, 0x0000,0x09be,0x257d,0x4f04};
const WORD SAWTOOTH_WAVE_TABLE[NUM_SAMPLES]={0x1000,0x2000,0x3000,0x4000, 0x5000,0x6000,0x7000,0x8000,
	0x8fff,0x9fff,0xafff,0xbfff, 0xcfff,0xdfff,0xefff,0xffff};
WORD samplesTable[NUM_SAMPLES];

//unsigned char ReceivedDataBuffer[64];
//unsigned char ToSendDataBuffer[64];
#define ReceivedDataBuffer hid_report_out
#define ToSendDataBuffer hid_report_in

USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;


volatile WORD tick10=0;
volatile BYTE second_10=0;
#ifdef USA_SW_RTC 
volatile PIC24_RTCC_DATE currentDate={{0,0,0}};
volatile PIC24_RTCC_TIME currentTime={{0,0,0}};
#else
PIC24_RTCC_DATE currentDate;
PIC24_RTCC_TIME currentTime;
#endif
volatile DWORD milliseconds;


struct SAVED_PARAMETERS configParms;

BYTE PLLlock;

#define TickGet() (tick10)

/** PRIVATE PROTOTYPES *********************************************/
static void InitializeSystem(void);
int UserInit(void);
void USBDeviceTasks(void);
void ProcessIO(void);
void sendEP1(void);
void USBCBSendResume(void);


BYTE setWave(void);

/** DECLARATIONS ***************************************************/
#pragma code



/********************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *******************************************************************/

int main(void) {   
	WORD i;

TRISD=0;
mLED_1=mLED_2=mLED_3=0;

mLED_1=1;

//	ecan1ClkInit(250000L);

	__delay_ms(50);		// vale *20~ non essendoci ancora PLL!
	ClrWdt();
  InitializeSystem();

	__delay_ms(100);
	ClrWdt();

mLED_2=1;

	/*TRISA=0;
	while(1) {
		LATA ^= 0xffff;
		ClrWdt();
		}*/


  DataEEInit();
	ClrWdt();
  dataEEFlags.val = 0;
  Nop();
  Nop();

	loadSettings();


	UserInit();
mLED_3=1;

  #if defined(USB_INTERRUPT)
    USBDeviceAttach();
  #endif


Beep();

	PLLlock=setWave();



  while(1) {

		ClrWdt();
//		LATB ^= 0xffff;


    #if defined(USB_POLLING)
// Check bus status and service USB interrupts.
#ifndef __DEBUG
  	USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
#endif
    				  // this function periodically.  This function will take care
    				  // of processing and responding to SETUP transactions 
    				  // (such as during the enumeration process when you first
    				  // plug in).  USB hosts require that USB devices should accept
    				  // and process SETUP packets in a timely fashion.  Therefore,
    				  // when using polling, this function should be called 
    				  // regularly (such as once every 1.8ms or faster** [see 
    				  // inline code comments in usb_device.c for explanation when
    				  // "or faster" applies])  In most cases, the USBDeviceTasks() 
    				  // function does not take very long to execute (ex: <100 
    				  // instruction cycles) before it returns.
    #endif
				  

  #if defined(USB_INTERRUPT)
    if(USB_BUS_SENSE && (USBGetDeviceState() == DETACHED_STATE)) { 
      USBDeviceAttach(); 
    	} 
    else    // ADDED BY ANGUEL 
    if(!USB_BUS_SENSE && (USBGetDeviceState() != DETACHED_STATE)) { 
      USBDeviceDetach(); 
//			USBDeviceState= DETACHED_STATE;
    	} 
  #endif


// Application-specific tasks.
// Application related code may be added here, or in the ProcessIO() function.
    ProcessIO();        

//	mLED_2_Toggle(); 



    if((USBDeviceState < CONFIGURED_STATE) && (USBDeviceState > ATTACHED_STATE)) {
//      continue; no...
    	}


		UserTasks();
	
	  } //end while
    
  
	} //end main

/********************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.                  
 *
 * Note:            None
 *******************************************************************/
static void InitializeSystem(void) {

	ClrWdt();

	LED0_TRIS=0; 	LED1_TRIS=0; LED2_TRIS=0;
//	BUTTON0_TRIS=1; BUTTON1_TRIS=1; BUTTON2_TRIS=1; per debug no..
		// 
	LATA = 0b0000000000000000;
	LATB = 0b0000000000000000;
	LATC = 0b0000000000000000;
	LATD = 0b0000000000000000;
	LATE = 0b0000000000000000;
	LATF = 0b0000000000000000;
	LATG = 0b0000000000000000;
	TRISA=0b1100000001000000;		// 3 tasti;
	TRISB=0b0000000000111100;		// 4 switch; 
	TRISC=0b0110000000000000;		// 2 tasti;
	TRISD=0b0000100100000001;		// 3 tasti; 3 led
	TRISE=0b0000000000000100;		// pwm; rs232 cts/rts
	TRISF=0b0011000100110100;		// 4 tasti; 
	TRISG=0b0010000000000000;		// 1 tasti; pwm; rs232
#if defined(__PIC24EP512GU810__)
	ANSELA=0x0000;
	ANSELB=0x0000;
	ANSELC=0x0000;
	ANSELD=0x0000;
	ANSELE=0x0000;
	ANSELG=0x0000;
#endif
	TRISGbits.TRISG15=0;		//buzzer


//	_PLLDIV=278 /*205*/;						// M = _PLLDIV+2, 2..513
//	_PLLPRE=6;						// N1 = _PLLPRE+2, 2..33
//	_PLLPOST=0;						// N2 = _PLLPOST, 2 (0), 4 (1), 8 (3)
	OSCTUN=0;			//(da USB)
// [Lavoro a 100MHz per ora (v .H anche, v. anche il blocco per dsPIC sopra) ]
// [(215*7.37)/(2*8*2) = 100~]
// (280*8)/(8*2) = 140

	PLLFBD = 68; // M = PLLFBD + 2 = 70
  CLKDIVbits.PLLPOST = 0; // N2 = 2
  CLKDIVbits.PLLPRE = 0; // N1 = 2
#warning overcloccare?? occhio a USB! (ma pare vada per fatti suoi)

/*	
  Fosc= Fin*M/(N1*N2)
	0.8MHz<(Fin/N1)<8MHz
	100MHz<(Fin*M/N1)<340MHz
	M=2,3,4,...513
	N1=2...33
	N2=2,4,8
	
	PLLFBD    = M -2;
	CLKDIVbits.PLLPRE = N1 -2;
	CLKDIVbits.PLLPOST = 0b00; //N2={2,4,R,8} */

//OSCCONbits.CLKLOCK=1;OSCCONbits.NOSC=1;OSCCONbits.OSWEN=1;
//  while (OSCCONbits.COSC != 0x7)LATB ^= 1;; 


	//Wait for the Primary PLL to lock and then
       // configure the auxiliary PLL to provide 48MHz needed for USB
       // Operation.
  while(OSCCONbits.LOCK != 1) 
		ClrWdt();			// boh?

// per USB
            // Configuring the auxiliary PLL, since the primary
            // oscillator provides the source clock to the auxiliary
            // PLL, the auxiliary oscillator is disabled. Note that
            // the AUX PLL is enabled. The input 8MHz clock is divided
            // by 2, multiplied by 24 and then divided by 2. Wait till
            // the AUX PLL locks.

            ACLKCON3 = 0x24c1 /*0x24C1*/;		// 8MHz input, diviso 2 e diviso 2
            ACLKDIV3 = 0x7;									// x24, ossia in tutto 48MHz!

#ifndef __DEBUG
            ACLKCON3bits.ENAPLL = 1;
            while(ACLKCON3bits.APLLCK != 1) ClrWdt();
#endif


  

//	The USB specifications require that USB peripheral devices must never source
//	current onto the Vbus pin.  Additionally, USB peripherals should not source
//	current on D+ or D- when the host/hub is not actively powering the Vbus line.
//	When designing a self powered (as opposed to bus powered) USB peripheral
//	device, the firmware should make sure not to turn on the USB module and D+
//	or D- pull up resistor unless Vbus is actively powered.  Therefore, the
//	firmware needs some means to detect when Vbus is being powered by the host.
//	A 5V tolerant I/O pin can be connected to Vbus (through a resistor), and
// 	can be used to detect when Vbus is high (host actively powering), or low
//	(host is shut down or otherwise not supplying power).  The USB firmware
// 	can then periodically poll this I/O pin to know when it is okay to turn on
//	the USB module/D+/D- pull up resistor.  When designing a purely bus powered
//	peripheral device, it is not possible to source current on D+ or D- when the
//	host is not actively providing power on Vbus. Therefore, implementing this
//	bus sense feature is optional.  This firmware can be made to use this bus
//	sense feature by making sure "USE_USB_BUS_SENSE_IO" has been defined in the
//	HardwareProfile.h file.    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
//	If the host PC sends a GetStatus (device) request, the firmware must respond
//	and let the host know if the USB peripheral device is currently bus powered
//	or self powered.  See chapter 9 in the official USB specifications for details
//	regarding this request.  If the peripheral device is capable of being both
//	self and bus powered, it should not return a hard coded value for this request.
//	Instead, firmware should check if it is currently self or bus powered, and
//	respond accordingly.  If the hardware has been configured like demonstrated
//	on the PICDEM FS USB Demo Board, an I/O pin can be polled to determine the
//	currently selected power source.  On the PICDEM FS USB Demo Board, "RA2" 
//	is used for	this purpose.  If using this feature, make sure "USE_SELF_POWER_SENSE_IO"
//	has been defined in HardwareProfile.h, and that an appropriate I/O pin has been mapped
//	to it in HardwareProfile.h.
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif



	
    //********* Initialize Peripheral Pin Select (PPS) *************************
    //  This section only pertains to devices that have the PPS capabilities.
    //    When migrating code into an application, please verify that the PPS
    //    setting is correct for the port pins that are used in the application.
#if defined(__PIC24EP512GU810__)
// non le trova... manco in XC... froci urfidi...@$%&
// http://www.microchip.com/forums/m801504.aspx  MBedder !
#define PPSIn(fn,pin)    iPPSInput(IN_FN_PPS##fn,IN_PIN_PPS##pin)
#define PPSOut(fn,pin)    iPPSOutput(OUT_PIN_PPS##pin,OUT_FN_PPS##fn)

//	PPSOut(_U1TX, _RP126);      // TXD 
//	PPSIn(_U1RX, _RP125);      // RXD 


	PPSOut(_OC2, _RP127);      // buzzer 4KHz , qua  rimappabile

	PPSOut(_OC1, _RP80);      // output (pwm cmq)


#endif


// non c'...	EnablePullUpCN20;			//pulsanti
	CNPUGbits.CNPUG6=1;
	CNPUCbits.CNPUC4=1;



	OpenTimer2(T2_ON & T2_IDLE_CON & T2_GATE_OFF & T2_32BIT_MODE_OFF & T2_PS_1_64 & T2_SOURCE_INT,
		TMR2BASE);		//10Hz per timing
	ConfigIntTimer2(T2_INT_PRIOR_4 & T2_INT_ON);
	EnableIntT2;
	OpenTimer4(T4_ON & T4_IDLE_CON & T4_GATE_OFF & T4_32BIT_MODE_ON & T4_PS_1_1 & T4_SOURCE_INT,
		TMR4BASE);		//1000000Hz per ev. timestamp e anche timebase PWM quando  basso
	ConfigIntTimer4(T2_INT_PRIOR_4 & T2_INT_OFF);
	DisableIntT4;


	{__asm__ volatile ("clrwdt");}		// xch qua ho disattivato! ma lo lascio al boot...
	



  CM3CON=0;		// comparatori off ?? boh


// buzzer 4KHz
  /* Reset PWM */
  OC2CON1 = 0x0000;
  OC2CON2 = 0x0000;
  
  /* set PWM duty cycle to 50% */
  OC2R    = FCY/2/BEEP_STD_FREQ/2  /* basato su SysClock => 4KHz circa ecc (16MHz / 256) */; //PWM_PERIOD >> 1; /* set the duty cycle tp 50% */
  OC2RS   = FCY/2/BEEP_STD_FREQ  /* se uso Timer come Src SEMBRA NON FARE NULLA... qua boh! */;  //PWM_PERIOD - 1;  /* set the period */
  
  /* configure PWM */
  OC2CON2 = 0x001f;   /* 0x001F = Sync with This OC module                               */
  OC2CON1 = 0x1c00 /* 0x0400 => src=Timer3 */;  /* 0x1C08 = Clock source Fcyc, trigger mode 1, Mode 0 (disable OC1) */
  
  /* enable the PWM */
  OC2CON1 |= 0x0006;   /* Mode 6, Edge-aligned PWM Mode */ //  v. beep


  OC1CON1 = 0x0000;
  OC1CON2 = 0x0000;
  
  OC1RS  = 32768;
  OC1R   = 16384;
  
  OC1CON2 = 0x001f;
  OC1CON1 = 0x1c00;
  
  OC1CON1 |= 0x0006;   /* Mode 6, Edge-aligned PWM Mode */ //  v. beep


#ifndef __DEBUG
  USBDeviceInit();
#endif


	} //end InitializeSystem


int UserInit(void) {

	ClrWdt();

	currentDate.mday=1;		//preset cmq
	currentDate.mon=1;

  //initialize the variable holding the handle for the last transmission
  USBOutHandle = 0;
  USBInHandle = 0;

	PLLlock=0;

	ClrWdt();
	return 1;
	}

void sendEP1(void) {

	USBInHandle = HIDTxPacket(HID_EP,(BYTE*)ToSendDataBuffer,64);
	while(HIDTxHandleBusy(USBInHandle))	{	// QUA DOPO! (xch il buffer  usato "live")
		ClrWdt();
		#if defined(USB_POLLING)
		USBDeviceTasks()
		#endif
		;
		}
	}



/********************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *
 * Note:            None
 *******************************************************************/
void ProcessIO(void) { 
	int i;

	ClrWdt();
//	mLED_1_Toggle(); 

  // User Application USB tasks
  if((USBDeviceState < CONFIGURED_STATE) || (USBSuspendControl==1)) {

		return;
		}


	if(!HIDRxHandleBusy(USBOutHandle)) {   
	  switch(ReceivedDataBuffer[1]) {			// ReceivedDataBuffer[0] contiene 2 = OUTPUT_REPORT, se ESPLICITAMENTE DEFINITO

			case CMD_NOP:
				break;
			
			case CMD_GETID:						// restituisce ID ASCII ()
				i=0;						// puntatore alla stringa Copyright
			
				do {
					prepOutBuffer(CMD_GETID);					// indicatore
					memcpy((void *)(ToSendDataBuffer+2),(void *)(((char *)CopyrString)+i),HID_INT_IN_EP_SIZE-2);
					sendEP1();
					i+=HID_INT_IN_EP_SIZE-2;
					} while(!memchr((void *)(ToSendDataBuffer+2),0,HID_INT_IN_EP_SIZE-2));
			
				break;
			
			case CMD_GETCAPABILITIES:								// restituisce config. 
				prepOutBuffer(CMD_GETCAPABILITIES);					// indicatore
				ToSendDataBuffer[2]=VERNUMH;
				ToSendDataBuffer[3]=VERNUML;
				ToSendDataBuffer[4]=0;		// marker per futuro...

				sendEP1();
				break;
			
			case CMD_GETCONFIG:											// restituisce config. 
				prepOutBuffer(CMD_GETCONFIG);					// indicatore

//				ToSendDataBuffer[4]=NodeID;
				sendEP1();
				break;
			
			case CMD_GETSTATUS:											
				prepOutBuffer(CMD_GETSTATUS);					// indicatore

				ToSendDataBuffer[4]=PLLlock;
				ToSendDataBuffer[8]=configParms.tipoOnda;
				ToSendDataBuffer[10]=LOBYTE(configParms.dutyCycle);
				ToSendDataBuffer[11]=HIBYTE(configParms.dutyCycle);
				ToSendDataBuffer[12]=LOBYTE(LOWORD(configParms.Frequenza));
				ToSendDataBuffer[13]=HIBYTE(LOWORD(configParms.Frequenza));
				ToSendDataBuffer[14]=LOBYTE(HIWORD(configParms.Frequenza));
				ToSendDataBuffer[15]=HIBYTE(HIWORD(configParms.Frequenza));
				ToSendDataBuffer[16]=configParms.Amplitude;

				sendEP1();

				break;

			case CMD_SETSTATUS:											// imposta
				prepOutBuffer(CMD_SETSTATUS);					// indicatore

	
				sendEP1();
				break;

			case CMD_SETCONFIG:											// leggi
				prepOutBuffer(CMD_SETCONFIG);					// indicatore

//				NodeID=ReceivedDataBuffer[4];

				sendEP1();
				break;

			case CMD_TEST:							// test: accendo/spengo led in base a b0
				prepOutBuffer(CMD_TEST);
				mLED_1=ReceivedDataBuffer[2] & 1;
				ToSendDataBuffer[2]=PORTA;		// tanto per
				sendEP1();
				break;

			case CMD_BEEP:
				prepOutBuffer(CMD_BEEP);
				Beep();			
				sendEP1();
				break;

			case CMD_INIT:											// 
				prepOutBuffer(CMD_INIT);					// indicatore

				configParms.tipoOnda=NESSUNA;
				configParms.dutyCycle=500;
				configParms.Frequenza=1000;			// tanto per
				configParms.Amplitude=50;

				ToSendDataBuffer[2]=PLLlock=setWave();
				sendEP1();
				break;

			case CMD_SETFUNCTION:											// 
				prepOutBuffer(CMD_SETFUNCTION);					// indicatore

				configParms.tipoOnda=ReceivedDataBuffer[8];
				configParms.dutyCycle=MAKEWORD(ReceivedDataBuffer[10],ReceivedDataBuffer[11]);
				configParms.Frequenza=MAKELONG(MAKEWORD(ReceivedDataBuffer[12],ReceivedDataBuffer[13]),MAKEWORD(ReceivedDataBuffer[14],ReceivedDataBuffer[15]));
				configParms.Amplitude=ReceivedDataBuffer[16];

				ToSendDataBuffer[2]=PLLlock=setWave();

				sendEP1();
				break;


			case CMD_WRITEEEPROM:					// programmazione EEprom (blocchi SEMPRE da 16 BYTE, con USB nuovo/2005):
				prepOutBuffer(CMD_WRITEEEPROM);			// indicatore

				i=MAKEWORD(ReceivedDataBuffer[3],ReceivedDataBuffer[4]);
				if(!i) {			// truschino :)
					saveSettings();
					ToSendDataBuffer[2]=1;
					}
				else {
					memcpy((void *)I2CBuffer,(void *)(ReceivedDataBuffer+5),ReceivedDataBuffer[2] & 0x1f);
					i=I2CWritePage16(i,ReceivedDataBuffer[2] & 0x1f);				// gestire ev. errore?
					ToSendDataBuffer[2]=i;
					}

				sendEP1();
				break;

			case CMD_READEEPROM:			// lettura EEprom: 2 BYTE lo/hi addr.
				prepOutBuffer(CMD_READEEPROM);			// indicatore

				i=MAKEWORD(ReceivedDataBuffer[3],ReceivedDataBuffer[4]);
				I2CRead16Seq(i,ReceivedDataBuffer[2] & 0x1f);			// 
				memcpy((void *)ToSendDataBuffer+2,(void *)I2CBuffer,ReceivedDataBuffer[2] & 0x1f);

				sendEP1();
				break;

			case CMD_READRTC:			// lettura RTC
				prepOutBuffer(CMD_READRTC);			// indicatore

#ifndef USA_SW_RTC 
        currentTime.l = PIC24RTCCGetTime();
        currentDate.l = PIC24RTCCGetDate();
		    ToSendDataBuffer[3]=from_bcd(currentDate.mday);		// UNIFORMARE CON SKYUSB!
		    ToSendDataBuffer[4]=from_bcd(currentDate.mon);
		    ToSendDataBuffer[5]=from_bcd(currentDate.year);
		    ToSendDataBuffer[6]=from_bcd(currentTime.hour);
		    ToSendDataBuffer[7]=from_bcd(currentTime.min);
		    ToSendDataBuffer[8]=from_bcd(currentTime.sec);
#else
  			ToSendDataBuffer[3]=currentDate.mday;		// UNIFORMARE CON SKYUSB!
		    ToSendDataBuffer[4]=currentDate.mon;
		    ToSendDataBuffer[5]=currentDate.year;
		    ToSendDataBuffer[6]=currentTime.hour;
		    ToSendDataBuffer[7]=currentTime.min;
		    ToSendDataBuffer[8]=currentTime.sec;
		
#endif
		 
				sendEP1();
				break;

			case CMD_WRITERTC:					// programmazione RTC 
				prepOutBuffer(CMD_WRITERTC);			// indicatore

				i=ReceivedDataBuffer[3];		// UNIFORMARE CON SKYUSB e GSMDEV!
		    currentDate.mday=i;
				i=ReceivedDataBuffer[4];
				currentDate.mon=i;
				i=ReceivedDataBuffer[5];
				currentDate.year=i;			// -2000
				i=ReceivedDataBuffer[6];
	  	  currentTime.hour=i;
				i=ReceivedDataBuffer[7];
				currentTime.min=i;
				i=ReceivedDataBuffer[8];
				currentTime.sec=i;
	
#ifndef USA_SW_RTC 
				PIC24RTCCSetDate(MAKEWORD(to_bcd(currentDate.year),0),MAKEWORD(to_bcd(currentDate.mday),to_bcd(currentDate.mon)));		// BCD...
				PIC24RTCCSetTime(MAKEWORD(to_bcd(currentTime.hour),0),MAKEWORD(to_bcd(currentTime.sec),to_bcd(currentTime.min)));		// BCD...
#endif

				sendEP1();
				break;


			case CMD_READTIMESTAMP:						// 
				prepOutBuffer(CMD_READTIMESTAMP);			// indicatore
				if(ReceivedDataBuffer[2]) {
					WriteTimer45(MAKELONG(MAKEWORD(ReceivedDataBuffer[4],ReceivedDataBuffer[5]),MAKEWORD(ReceivedDataBuffer[6],ReceivedDataBuffer[7])));
					}
				{
				DWORD tsc;
				tsc=ReadTimer45();
  			ToSendDataBuffer[4]=LOBYTE(tsc);
  			ToSendDataBuffer[5]=HIBYTE(tsc);
  			ToSendDataBuffer[6]=LOBYTE(tsc);
  			ToSendDataBuffer[7]=HIBYTE(tsc);
				}
				sendEP1();
				break;

			case CMD_READ_IDLOCS:						// 
				{
					WORD LW,HW;
					TBLPAG = 0xFF;

					LW = __builtin_tblrdl(0);
					HW = __builtin_tblrdh(2);

				prepOutBuffer(CMD_DEBUG_ECHO);
  			ToSendDataBuffer[4]=LOBYTE(LW);
  			ToSendDataBuffer[5]=HIBYTE(LW);
  			ToSendDataBuffer[6]=LOBYTE(HW);
  			ToSendDataBuffer[7]=HIBYTE(HW);
				sendEP1();
				}
				break;	

			case CMD_RESET_IF:						// RESET (occhio a Enum. USB)
				Reset();
				break;	
						
			case CMD_DEBUG_ECHO:						// ECHO
				prepOutBuffer(CMD_DEBUG_ECHO);
				memcpy((void *)(ToSendDataBuffer+2),(void *)(ReceivedDataBuffer+2),
			//						min(HID_INT_IN_EP_SIZE,HID_INT_OUT_EP_SIZE)-2);
					HID_INT_OUT_EP_SIZE-2);
			
				sendEP1();
				break;
			}
	  USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
 		}

 
	}	//end ProcessIO


void prepOutBuffer(BYTE t) {					// entra W=IDENTificatore

	clearOutBuffer();
	ToSendDataBuffer[0]=1 /*SKYNET_REPORT_ID_INPUT*/;					// dev'essere per forza 1! (ossia ID report di input)
											 													// .. contiene xxx_REPORT_ID_INPUT, se metto esplicitamente un REPORT_ID
	ToSendDataBuffer[1]=t;
	}

void clearOutBuffer(void) {

	memset(ToSendDataBuffer,0,64 /*BUF_SIZE_USB*/);
	}

void prepStatusBuffer(BYTE t) {

	prepOutBuffer(CMD_GETSTATUS);				// indicatore
	ToSendDataBuffer[2]=t;
	}

void UserTasks(void) {
	static BYTE oldSw=3;
	static WORD cnt=0,cnt2=0;


	if(second_10) {
		second_10=0;

		cnt++;
		cnt2++;

		mLED_2^=1;


		if(cnt2>=8) {
			cnt2=0;
			if(!PORTFbits.RF5) {		// TEST MESSAGGI USB
				prepOutBuffer(CMD_TEST);					// indicatore
				ToSendDataBuffer[4]=PORTA;
				ToSendDataBuffer[5]=sw1;
				ToSendDataBuffer[6]=sw2;
				sendEP1();
				}
			}

		oldSw=(sw1 ? 1 : 0) | (sw2 ? 2 : 0);

		}		// second_1


//nonsleep:;
	}


BYTE setWave(void) {
	WORD n;
	int i;
	BYTE retVal=1;

	switch(configParms.tipoOnda) {
		case NESSUNA:
		  OC1CON1 &= ~0x0006;
			DisableIntT3;
			break;

		case QUADRA:
			if(configParms.Frequenza>=350000000L) {			//70MHz clock=35MHz max freq
// si potrebbe usare REFCLK??
	PPSUnLock;
	PPSOut(_REFCLKO, _RP80);      // RefClk 
	REFOCONbits.ROSSLP=1;
	REFOCONbits.ROSEL=1;
	REFOCONbits.RODIV=0;			// 0=>70MHz, 1=35
	REFOCONbits.ROON=1;
	TRISGbits.TRISG0=1;
	PPSLock;

				retVal=0;
				}	
			if(configParms.Frequenza<42L) {			//4.2Hz min freq con prescaler 256.. usare sw pwm?
// si potrebbe usare REFCLK?? mandandolo in ingresso di timer3
	PPSUnLock;
	PPSOut(_REFCLKO, _RP65);      // RefClk , a caso RD1
	PPSIn(_T3CK, _RP65);      // Timer 3 input
	REFOCONbits.ROSSLP=1;
	REFOCONbits.ROSEL=1;
	REFOCONbits.RODIV=0b1100;			// 4096 => 17089Hz
	REFOCONbits.ROON=1;
	TRISDbits.TRISD0=0;
	PPSLock;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_EXT,
					1000 /*non usato!*/);

				retVal=0;
				}	
			if(configParms.Frequenza>=20000) {			//2KHz!
				DisableIntT3;
			  n = (10*FCY)/configParms.Frequenza;
			  OC1R    = n/2;
			  OC1RS   = n;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x1c00;

				OpenTimer3(T3_OFF & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,0);
				DisableIntT3;
				}
			else if(configParms.Frequenza>=2000) {
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,
					1000 /*non usato!*/);
				DisableIntT3;
			  n = (10*FCY/8)/configParms.Frequenza;
			  OC1R    = n/2;
			  OC1RS   = n;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x0400 /* 0x0400 => src=Timer3 */;
				}	
			else if(configParms.Frequenza>=200) {
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_64 & T3_SOURCE_INT,
					1000 /*non usato!*/);
				DisableIntT3;
			  n = (10*FCY/64)/configParms.Frequenza;
			  OC1R    = n/2;
			  OC1RS   = n;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x0400 /* 0x0400 => src=Timer3 */;
				}	
			else {
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_256 & T3_SOURCE_INT,
					1000 /*non usato!*/);
				DisableIntT3;
			  n = (10*FCY/256)/configParms.Frequenza;
			  OC1R    = n/2;
			  OC1RS   = n;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x0400 /* 0x0400 => src=Timer3 */;
				}	
  
  		OC1CON1 |= 0x0006;
			break;

		case TRIANGOLARE:
			if(configParms.Frequenza>=5000000L) {			//diciamo che oltre i 500KHz non ci si va in C: fanno 3.2MHz di interrupt...
				retVal=0;
				}	
			if(configParms.Frequenza>=1000000L) {			//oltre 100KHz passo a 8samples
			  n = (10*FCY/8)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES/2; i++)
					samplesTable[i]=TRIANGLE_WAVE_TABLE[i*2] >> 12;
				for(i=0; i<NUM_SAMPLES/2; i++)
					samplesTable[i+NUM_SAMPLES/2]=TRIANGLE_WAVE_TABLE[i*2] >> 12;
			  OC1RS    = 16;		//4MHz
				}
			else if(configParms.Frequenza>=200000L) {			//20KHz!
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=TRIANGLE_WAVE_TABLE[i] >> 12;
			  OC1RS    = 16;		//4MHz
				}
			else if(configParms.Frequenza>=20000) {
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=TRIANGLE_WAVE_TABLE[i] >> 8;
			  OC1RS    = 256;		//275KHz
				}
			else if(configParms.Frequenza>=2000) {
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=TRIANGLE_WAVE_TABLE[i] >> 8;
			  OC1RS    = 256;		//275KHz
				}	
			else if(configParms.Frequenza>=200) {
			  n = (10*FCY/128)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=TRIANGLE_WAVE_TABLE[i] >> 4;
			  OC1RS    = 4096;		//17KHz
				}	
			else {
			  n = (10*FCY/128)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=TRIANGLE_WAVE_TABLE[i] >> 4;
			  OC1RS    = 4096;		//17KHz
				}	
  
			EnableIntT3;
		  OC1CON1 = 0x1c00;			//70MHz
  		OC1CON1 |= 0x0006;
			break;

		case SAWTOOTH:
			if(configParms.Frequenza>=5000000L) {			//diciamo che oltre i 500KHz non ci si va in C: fanno 3.2MHz di interrupt...
				retVal=0;
				}	
			if(configParms.Frequenza>=1000000L) {			//oltre 100KHz passo a 8samples
			  n = (10*FCY/8)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES/2; i++)
					samplesTable[i]=SAWTOOTH_WAVE_TABLE[i*2] >> 12;
				for(i=0; i<NUM_SAMPLES/2; i++)
					samplesTable[i+NUM_SAMPLES/2]=SAWTOOTH_WAVE_TABLE[i*2] >> 12;
			  OC1RS    = 16;		//4MHz
				}
			else if(configParms.Frequenza>=200000L) {			//20KHz!
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SAWTOOTH_WAVE_TABLE[i] >> 12;
			  OC1RS    = 16;		//4MHz
				}
			else if(configParms.Frequenza>=20000) {
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SAWTOOTH_WAVE_TABLE[i] >> 8;
			  OC1RS    = 256;		//275KHz
				}
			else if(configParms.Frequenza>=2000) {
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SAWTOOTH_WAVE_TABLE[i] >> 8;
			  OC1RS    = 256;		//275KHz
				}	
			else if(configParms.Frequenza>=200) {
			  n = (10*FCY/128)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SAWTOOTH_WAVE_TABLE[i] >> 4;
			  OC1RS    = 4096;		//17KHz
				}	
			else {
			  n = (10*FCY/128)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SAWTOOTH_WAVE_TABLE[i] >> 4;
			  OC1RS    = 4096;		//17KHz
				}	
  
			EnableIntT3;
		  OC1CON1 = 0x1c00;
  		OC1CON1 |= 0x0006;
			break;

		case SINUSOIDALE:
			if(configParms.Frequenza>=5000000L) {			//diciamo che oltre i 500KHz non ci si va in C: fanno 3.2MHz di interrupt...
				retVal=0;
				}	
			if(configParms.Frequenza>=1000000L) {			//oltre 100KHz passo a 8samples
			  n = (10*FCY/8)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES/2; i++)
					samplesTable[i]=SINE_WAVE_TABLE[i*2] >> 12;
				for(i=0; i<NUM_SAMPLES/2; i++)
					samplesTable[i+NUM_SAMPLES/2]=SINE_WAVE_TABLE[i*2] >> 12;
			  OC1RS    = 16;		//4MHz
				}
			else if(configParms.Frequenza>=200000L) {			//20KHz!
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SINE_WAVE_TABLE[i] >> 12;
			  OC1RS    = 16;		//4MHz
				}
			else if(configParms.Frequenza>=20000) {
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SINE_WAVE_TABLE[i] >> 8;
			  OC1RS    = 256;		//275KHz
				}
			else if(configParms.Frequenza>=2000) {
			  n = (10*FCY/16)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SINE_WAVE_TABLE[i] >> 8;
			  OC1RS    = 256;		//275KHz
				}	
			else if(configParms.Frequenza>=200) {
			  n = (10*FCY/128)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SINE_WAVE_TABLE[i] >> 4;
			  OC1RS    = 4096;		//17KHz
				}	
			else {
			  n = (10*FCY/128)/configParms.Frequenza;
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,n);
				for(i=0; i<NUM_SAMPLES; i++)
					samplesTable[i]=SINE_WAVE_TABLE[i] >> 4;
			  OC1RS    = 4096;		//17KHz
				}	
  
			EnableIntT3;
		  OC1CON1 = 0x1c00;
  		OC1CON1 |= 0x0006;
			break;

		case IMPULSI:
			if(configParms.Frequenza>=350000000L) {			//70MHz clock=35MHz max freq
				retVal=0;
				}	
			if(configParms.Frequenza<42L) {			//4.2Hz min freq con prescaler 256.. usare sw pwm?
				retVal=0;
				}	
			if(configParms.dutyCycle==0 || configParms.dutyCycle>=1000) {		// tanto per...!
				retVal=0;
				}	
			if(configParms.Frequenza>=20000) {			//2KHz!
				DisableIntT3;
			  n = (10*FCY)/configParms.Frequenza;
			  OC1RS    = n;
			  OC1R   = (((DWORD)n)*configParms.dutyCycle)/1000;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x1c00;
				}
			else if(configParms.Frequenza>=2000) {
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_8 & T3_SOURCE_INT,
					1000 /*non usato!*/);
				DisableIntT3;
			  n = (10*FCY/8)/configParms.Frequenza;
			  OC1RS    = n;
			  OC1R   = (((DWORD)n)*configParms.dutyCycle)/1000;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x0400 /* 0x0400 => src=Timer3 */;
				}	
			else if(configParms.Frequenza>=200) {
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_64 & T3_SOURCE_INT,
					1000 /*non usato!*/);
				DisableIntT3;
			  n = (10*FCY/64)/configParms.Frequenza;
			  OC1RS    = n;
			  OC1R   = (((DWORD)n)*configParms.dutyCycle)/1000;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x0400 /* 0x0400 => src=Timer3 */;
				}	
			else {
				OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_256 & T3_SOURCE_INT,
					1000 /*non usato!*/);
				DisableIntT3;
			  n = (10*FCY/256)/configParms.Frequenza;
			  OC1RS    = n;
			  OC1R   = (((DWORD)n)*configParms.dutyCycle)/1000;
			  
			  OC1CON2 = 0x001f;
			  OC1CON1 = 0x0400 /* 0x0400 => src=Timer3 */;
				}	
  
  		OC1CON1 |= 0x0006;
			break;

		case RUMORE_BIANCO:
			OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,
				TMR3BASE_2);			// cos a caso
			EnableIntT3;
		  n = 64 /*FCY/1000000L*/;			// 1MHz
		  OC1RS    = n;
		  OC1R   = n/2;
  		OC1CON1 |= 0x0006;
			break;

		case RUMORE_ROSA:
			OpenTimer3(T3_ON & T3_IDLE_CON & T3_GATE_OFF & T3_PS_1_1 & T3_SOURCE_INT,
				TMR3BASE_2);
			EnableIntT3;
		  n = 64 /*FCY/1000000L*/;			// 1MHz.. cambiare per fare rosa?
		  OC1RS    = n;
		  OC1R   = n/2;
  		OC1CON1 |= 0x0006;
			break;
		}

	return retVal;
	}


void Beep(void) {

  OC2R    = FCY/2/BEEP_STD_FREQ/2;
  OC2RS   = FCY/2/BEEP_STD_FREQ;

  OC2CON1 |= 0x0006;   /* Mode 6, Edge-aligned PWM Mode */
	__delay_ms(100);
	ClrWdt();
	__delay_ms(100);
	ClrWdt();
	__delay_ms(100);
	ClrWdt();
  OC2CON1 &= ~0x0006; 
	}

void saveSettings(void) {

	memcpy(I2CBuffer,(unsigned char*)&configParms, sizeof(struct SAVED_PARAMETERS));
	I2CWritePage16(0,sizeof(I2CBuffer));				// 
	}

void loadSettings(void) {

rifo:
	I2CRead16Seq(0,16);			// 
	memcpy((unsigned char*)&configParms,I2CBuffer,16 /* solo la parte iniziale! */);
	if(configParms.signature != 0x4443) {
		resetSettings();
		goto rifo;
		}

	I2CRead16Seq(0,sizeof(I2CBuffer));			// 
	memcpy((unsigned char*)&configParms,I2CBuffer,sizeof(struct SAVED_PARAMETERS));
	}

void resetSettings(void) {
	int i;
	BYTE *p,*p2;
	extern const struct SAVED_PARAMETERS DefaultParameters;

/*  p=(BYTE *)&DefaultParameters;
	p2=(BYTE *)&configParms;
	for(i=0; i<sizeof(struct SAVED_PARAMETERS); i++) {
 		EEscrivi_(i,*p++);
		} */
	memcpy(I2CBuffer,(unsigned char*)&DefaultParameters, sizeof(struct SAVED_PARAMETERS));
	I2CWritePage16(0,sizeof(I2CBuffer));				// 


	for(i=0; i<3; i++) {
		mLED_1=mLED_2=mLED_3=1;
		__delay_ms(200);
		ClrWdt();
		mLED_1=mLED_2=mLED_3=0;
		__delay_ms(200);
		ClrWdt();
		}

	}


/****************************************************************************
  Function:
    DWORD   PIC24RTCCGetDate( void )

  Description:
    This routine reads the date from the RTCC module.

  Precondition:
    The RTCC module has been initialized.


  Parameters:
    None

  Returns:
    DWORD in the format <xx><YY><MM><DD>

  Remarks:
    To catch roll-over, we do two reads.  If the readings match, we return
    that value.  If the two do not match, we read again until we get two
    matching values.

    For the PIC32MX, we use library routines, which return the date in the
    PIC32MX format.
  ***************************************************************************/

DWORD PIC24RTCCGetDate( void ) {
  DWORD_VAL   date1;
  DWORD_VAL   date2;

  do {
    while(RCFGCALbits.RTCSYNC);

    RCFGCALbits.RTCPTR0 = 1;
    RCFGCALbits.RTCPTR1 = 1;
    date1.w[1] = RTCVAL;
    date1.w[0] = RTCVAL;

    RCFGCALbits.RTCPTR0 = 1;
    RCFGCALbits.RTCPTR1 = 1;
    date2.w[1] = RTCVAL;
    date2.w[0] = RTCVAL;

    } while (date1.Val != date2.Val);

  return date1.Val;
	}

/****************************************************************************
  Function:
    DWORD   PIC24RTCCGetTime( void )

  Description:
    This routine reads the time from the RTCC module.

  Precondition:
    The RTCC module has been initialized.

  Parameters:
    None

  Returns:
    DWORD in the format <xx><HH><MM><SS>

  Remarks:
    To catch roll-over, we do two reads.  If the readings match, we return
    that value.  If the two do not match, we read again until we get two
    matching values.

    For the PIC32MX, we use library routines, which return the time in the
    PIC32MX format.
  ***************************************************************************/

DWORD PIC24RTCCGetTime( void ) {
  DWORD_VAL   time1;
  DWORD_VAL   time2;

  do {
    while(RCFGCALbits.RTCSYNC);

    RCFGCALbits.RTCPTR0 = 1;
    RCFGCALbits.RTCPTR1 = 0;
    time1.w[1] = RTCVAL;
    time1.w[0] = RTCVAL;

    RCFGCALbits.RTCPTR0 = 1;
    RCFGCALbits.RTCPTR1 = 0;
    time2.w[1] = RTCVAL;
    time2.w[0] = RTCVAL;

    } while (time1.Val != time2.Val);

    return time1.Val;
	}


/****************************************************************************
  Function:
    void PIC24RTCCSetDate( WORD xx_year, WORD month_day )

  Description:
    This routine sets the RTCC date to the specified value.


  Precondition:
    The RTCC module has been initialized.

  Parameters:
    WORD xx_year    - BCD year in the lower byte
    WORD month_day  - BCD month in the upper byte, BCD day in the lower byte

  Returns:
    None

  Remarks:
    For the PIC32MX, we use library routines.
  ***************************************************************************/

void PIC24RTCCSetDate( WORD xx_year, WORD month_day ) {
  
	UnlockRTCC();
  RCFGCALbits.RTCPTR0 = 1;
  RCFGCALbits.RTCPTR1 = 1;
  RTCVAL = xx_year;
  RTCVAL = month_day;
	}


/****************************************************************************
  Function:
    void PIC24RTCCSetTime( WORD weekDay_hours, WORD minutes_seconds )

  Description:
    This routine sets the RTCC time to the specified value.

  Precondition:
    The RTCC module has been initialized.

  Parameters:
    WORD weekDay_hours      - BCD weekday in the upper byte, BCD hours in the
                                lower byte
    WORD minutes_seconds    - BCD minutes in the upper byte, BCD seconds in
                                the lower byte

  Returns:
    None

  Remarks:
    For the PIC32MX, we use library routines.
  ***************************************************************************/

void PIC24RTCCSetTime( WORD weekDay_hours, WORD minutes_seconds) {

  UnlockRTCC();
  RCFGCALbits.RTCPTR0 = 1;
  RCFGCALbits.RTCPTR1 = 0;
  RTCVAL = weekDay_hours;
  RTCVAL = minutes_seconds;
	}

/****************************************************************************
  Function:
    void UnlockRTCC( void )

  Description:
    This function unlocks the RTCC so we can write a value to it.

  Precondition:
    None

  Parameters:
    None

  Return Values:
    None

  Remarks:
    For the PIC32MX, we use library routines.
  ***************************************************************************/

#define RTCC_INTERRUPT_REGISTER IEC3
#define RTCC_INTERRUPT_VALUE    0x2000

void UnlockRTCC( void ) {
  BOOL interruptsWereOn;

  interruptsWereOn = FALSE;
  if((RTCC_INTERRUPT_REGISTER & RTCC_INTERRUPT_VALUE) == RTCC_INTERRUPT_VALUE) {
    interruptsWereOn = TRUE;
    RTCC_INTERRUPT_REGISTER &= ~RTCC_INTERRUPT_VALUE;
    }

  // Unlock the RTCC module
  __asm__ ("mov #NVMKEY,W0");
  __asm__ ("mov #0x55,W1");
  __asm__ ("mov #0xAA,W2");
  __asm__ ("mov W1,[W0]");
  __asm__ ("nop");
  __asm__ ("mov W2,[W0]");
  __asm__ ("bset RCFGCAL,#13");
  __asm__ ("nop");
  __asm__ ("nop");

  if(interruptsWereOn) {
    RTCC_INTERRUPT_REGISTER |= RTCC_INTERRUPT_VALUE;
    }
	}


BYTE to_bcd(BYTE n) {
	
	return (n % 10) | ((n / 10) << 4);
	}

BYTE from_bcd(BYTE n) {
	
	return (n & 15) + (10*(n >> 4));
	}




// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Call back that is invoked when a USB suspend is detected
 *
 * Note:            None
 *****************************************************************************/
void USBCBSuspend(void) {

	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
	//things to not work as intended.	
	

  #if defined(__C30__)
		_IPL=1; //MAL 3/12			*********** con ethernet questa fa piantare tutto!!!
#ifdef USA_SLEEP
    MyUSBSleepOnSuspend();
#endif
  #endif
	}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 *
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void) {
  #if !defined(self_powered)
      if(U1OTGIRbits.ACTVIF) {
        IEC5bits.USB1IE = 0;
        U1OTGIEbits.ACTVIE = 0;
        IFS5bits.USB1IF = 0;
      
      //USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
      USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
      //USBSuspendControl = 0;
      }
  #endif
	}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup from USB suspend 
 *					is detected.
 *
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend(void) {
	
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be 
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).

//	esciSleep(1); mah, 2014
	}

/********************************************************************
 Function:        void USBCB_SOF_Handler(void)

 Description:     The USB host sends out a SOF packet to full-speed
                  devices every 1 ms. This interrupt may be useful
                  for isochronous pipes. End designers should
                  implement callback routine as necessary.

 PreCondition:    None
 
 Parameters:      None
 
 Return Values:   None
 
 Remarks:         None
 *******************************************************************/
void USBCB_SOF_Handler(void) {
  // No need to clear UIRbits.SOFIF to 0 here.
  // Callback caller is already doing that.
	}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler(void) {
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
	}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq(void) {

  USBCheckHIDRequest();
	}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 *
 * Note:            None
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
    // Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 *
 * Note:            None
 *******************************************************************/
void USBCBInitEP(void) {

  //enable the HID endpoint
  USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
  //Re-arm the OUT endpoint for the next packet
  USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);

	}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *					
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes 
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function will only be able to wake up the host if
 *                  all of the below are true:
 *					
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET 
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.   
 *
 *                  If the host has not armed the device to perform remote wakeup,
 *                  then this function will return without actually performing a
 *                  remote wakeup sequence.  This is the required behavior, 
 *                  as a USB device that has not been armed to perform remote 
 *                  wakeup must not drive remote wakeup signalling onto the bus;
 *                  doing so will cause USB compliance testing failure.
 *                  
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            This function does nothing and returns quickly, if the USB
 *                  bus and host are not in a suspended condition, or are 
 *                  otherwise not in a remote wakeup ready state.  Therefore, it
 *                  is safe to optionally call this function regularly, ex: 
 *                  anytime application stimulus occurs, as the function will
 *                  have no effect, until the bus really is in a state ready
 *                  to accept remote wakeup. 
 *
 *                  When this function executes, it may perform clock switching,
 *                  depending upon the application specific code in 
 *                  USBCBWakeFromSuspend().  This is needed, since the USB
 *                  bus will no longer be suspended by the time this function
 *                  returns.  Therefore, the USB module will need to be ready
 *                  to receive traffic from the host.
 *
 *                  The modifiable section in this routine may be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of ~3-15 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at least 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void) {
  static WORD delay_count;
    
    //First verify that the host has armed us to perform remote wakeup.
    //It does this by sending a SET_FEATURE request to enable remote wakeup,
    //usually just before the host goes to standby mode (note: it will only
    //send this SET_FEATURE request if the configuration descriptor declares
    //the device as remote wakeup capable, AND, if the feature is enabled
    //on the host (ex: on Windows based hosts, in the device manager 
    //properties page for the USB device, power management tab, the 
    //"Allow this device to bring the computer out of standby." checkbox 
    //should be checked).
    if(USBGetRemoteWakeupStatus() == TRUE) {
        //Verify that the USB bus is in fact suspended, before we send
        //remote wakeup signalling.
      if(USBIsBusSuspended() == TRUE) {
        USBMaskInterrupts();
            
        //Clock switch to settings consistent with normal USB operation.
        USBCBWakeFromSuspend();
        USBSuspendControl = 0; 
        USBBusIsSuspended = FALSE;  //So we don't execute this code again, 
                                        //until a new suspend condition is detected.

        //Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
        //device must continuously see 5ms+ of idle on the bus, before it sends
        //remote wakeup signalling.  One way to be certain that this parameter
        //gets met, is to add a 2ms+ blocking delay here (2ms plus at 
        //least 3ms from bus idle to USBIsBusSuspended() == TRUE, yeilds
        //5ms+ total delay since start of idle).
        delay_count = 3600U;        
        do {
          delay_count--;
          } while(delay_count);
            
        //Now drive the resume K-state signalling onto the USB bus.
        USBResumeControl = 1;       // Start RESUME signaling
        delay_count = 1800U;        // Set RESUME line for 1-13 ms
        do {
          delay_count--;
        	} while(delay_count);
        USBResumeControl = 0;       //Finished driving resume signalling

        USBUnmaskInterrupts();
        }
    }
	}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * PreCondition:    None
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size) {

  switch(event) {
    case EVENT_TRANSFER:
      //Add application specific callback task or callback function here if desired.
      break;
    case EVENT_SOF:
      USBCB_SOF_Handler();
      break;
    case EVENT_SUSPEND:
      USBCBSuspend();
      break;
    case EVENT_RESUME:
      USBCBWakeFromSuspend();
      break;
    case EVENT_CONFIGURED: 
      USBCBInitEP();
      break;
    case EVENT_SET_DESCRIPTOR:
      USBCBStdSetDscHandler();
      break;
    case EVENT_EP0_REQUEST:
      USBCBCheckOtherReq();
      break;
  	case EVENT_BUS_ERROR:
      USBCBErrorHandler();
      break;
	  case EVENT_TRANSFER_TERMINATED:
	      //Add application specific callback task or callback function here if desired.
	      //The EVENT_TRANSFER_TERMINATED event occurs when the host performs a CLEAR
	      //FEATURE (endpoint halt) request on an application endpoint which was 
	      //previously armed (UOWN was = 1).  Here would be a good place to:
	      //1.  Determine which endpoint the transaction that just got terminated was 
	      //      on, by checking the handle value in the *pdata.
	      //2.  Re-arm the endpoint if desired (typically would be the case for OUT 
	      //      endpoints).
	
	      //Check if the host recently did a clear endpoint halt on the MSD OUT endpoint.
	      //In this case, we want to re-arm the MSD OUT endpoint, so we are prepared
	      //to receive the next CBW that the host might want to send.
	      //Note: If however the STALL was due to a CBW not valid condition, 
	      //then we are required to have a persistent STALL, where it cannot 
	      //be cleared (until MSD reset recovery takes place).  See MSD BOT 
	      //specs v1.0, section 6.6.1.

	    break;
	  default:
	    break;
    }

  return TRUE; 
	}

const struct SAVED_PARAMETERS DefaultParameters =
	{ 
	0x4443, 	// signature
	QUADRA,
	500,
	10000,
	100
	};

void EEscrivi_(BYTE addr,BYTE n) {	// 
  
  DataEEWrite(n,addr);
  }
BYTE EEleggi(BYTE addr) {
  
  return DataEERead(addr);
  }
